package quimufu.colourful_portals.portal;

import dev.onyxstudios.cca.api.v3.component.Component;
import net.minecraft.nbt.NbtCompound;
import net.minecraft.nbt.NbtElement;
import net.minecraft.nbt.NbtHelper;
import net.minecraft.nbt.NbtList;
import net.minecraft.util.Identifier;
import net.minecraft.util.math.BlockBox;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.WorldProperties;
import quimufu.colourful_portals.util.Pair;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import static quimufu.colourful_portals.ColourfulPortalsMod.LOGGER;

public class PortalListComponent implements Component {
    private final WorldProperties worldProperties;
    HashMap<Identifier, List<Pair<BlockBox, Identifier>>> portalsPerPortalBlock = new HashMap<>();

    public PortalListComponent(WorldProperties worldProperties) {
        this.worldProperties = worldProperties;
    }

    @Override
    public void readFromNbt(NbtCompound tag) {
        if (tag != null && !tag.isEmpty()) {
            for (String block : tag.getKeys()) {
                NbtList portalsWithDim = tag.getList(block, NbtElement.COMPOUND_TYPE);
                ArrayList<Pair<BlockBox, Identifier>> portals = new ArrayList<>();
                for (int i = 0; i < portalsWithDim.size(); i++) {
                    NbtCompound portalWithDim = portalsWithDim.getCompound(i);

                    NbtCompound portalCompound = portalWithDim.getCompound("portal");
                    BlockBox portal = BlockBox.create(NbtHelper.toBlockPos(portalCompound.getCompound("from")),
                            NbtHelper.toBlockPos(portalCompound.getCompound("to")));

                    Identifier dimension = Identifier.tryParse(portalWithDim.getString("dim"));

                    portals.add(Pair.of(portal, dimension));
                }
                portalsPerPortalBlock.put(Identifier.tryParse(block), portals);
            }
        }
    }

    @Override
    public void writeToNbt(NbtCompound tag) {
        for (Identifier portalBlockId : portalsPerPortalBlock.keySet()) {
            NbtList portalsWithDimList = new NbtList();
            tag.put(portalBlockId.toString(), portalsWithDimList);
            for (Pair<BlockBox, Identifier> portalWithDim : portalsPerPortalBlock.get(portalBlockId)) {
                NbtCompound portalWithDimCompound = new NbtCompound();
                portalsWithDimList.add(portalWithDimCompound);

                NbtCompound portalCompound = new NbtCompound();
                portalWithDimCompound.put("portal", portalCompound);
                portalWithDimCompound.putString("dim", portalWithDim.second.toString());

                BlockBox portal = portalWithDim.first;
                BlockPos from = new BlockPos(portal.getMinX(), portal.getMinY(), portal.getMinZ());
                BlockPos to = new BlockPos(portal.getMaxX(), portal.getMaxY(), portal.getMaxZ());

                portalCompound.put("from", NbtHelper.fromBlockPos(from));
                portalCompound.put("to", NbtHelper.fromBlockPos(to));
            }

        }
        LOGGER.debug("portals {}", tag.toString());


    }

    public List<BlockBox> getContainingPortals(Identifier blockId, BlockPos pos, Identifier dim) {
        return portalsPerPortalBlock.computeIfAbsent(blockId, (i) -> new ArrayList<>()).stream()
                .filter((portalWithDim) -> (portalWithDim.second.equals(dim)
                        && portalWithDim.first.contains(pos)))
                .map((p) -> p.first)
                .toList();
    }

    public void createPortal(Identifier blockId, Pair<BlockBox, Identifier> portalWithDim) {
        List<Pair<BlockBox, Identifier>> portals = portalsPerPortalBlock.computeIfAbsent(blockId, (i) -> new ArrayList<>());
        if (!portals.contains(portalWithDim)) {
            portals.add(portalWithDim);
        }
    }

    public void removePortal(Identifier blockId, Pair<BlockBox, Identifier> portalWithDim) {
        List<Pair<BlockBox, Identifier>> portals = portalsPerPortalBlock.computeIfAbsent(blockId, (i) -> new ArrayList<>());
        portals.remove(portalWithDim);
    }

    public List<Pair<BlockBox, Identifier>> getPortals(Identifier blockId) {
        return portalsPerPortalBlock.computeIfAbsent(blockId, (i) -> new ArrayList<>());
    }


    public Set<Identifier> getBlockIds() {
        return portalsPerPortalBlock.keySet();
    }

    public Pair<BlockBox, Identifier> getNext(Identifier blockId, BlockBox portal, Identifier dim) {
        List<Pair<BlockBox, Identifier>> portals = portalsPerPortalBlock.computeIfAbsent(blockId, (i) -> new ArrayList<>());
        Pair<BlockBox, Identifier> portalWithDim = Pair.of(portal, dim);

        int nextIndex = portals.indexOf(portalWithDim) + 1;
        if (nextIndex >= portals.size()) {
            return portals.get(0);
        }

        return portals.get(nextIndex);
    }

    public boolean containsPortal(Identifier blockId, Pair<BlockBox,Identifier> portal) {
        List<Pair<BlockBox, Identifier>> portals = portalsPerPortalBlock.computeIfAbsent(blockId, (i) -> new ArrayList<>());
        return portals.contains(portal);
    }

    public Pair<BlockBox, Identifier>  getLast(Identifier blockId) {
        List<Pair<BlockBox, Identifier>> portals = portalsPerPortalBlock.computeIfAbsent(blockId, (i) -> new ArrayList<>());
        return portals.get(portals.size()-1);
    }
}
